interval = 10 -- cycle divided by interval must result in an integer
cycle = 240 -- this should always be a multiple of 8 (as 7.5 ore/s is one ore per 8 ticks)
batches = cycle / interval -- the quotient must be an integer
insertcount = cycle / 8 -- the quotient must be an integer

script.on_init(function()
	global.siphons = {}
	global.siphons_index = {}
	global.highlight = nil
	global.siphon_range = {}
end)


function siphonbuilt(event)
	local ent = event.created_entity
	if not ent then return end -- only undefined if a naughty script doesn't populate it. This will break this mod if an ore siphon is constructed with a malformed events.script_raised_built
	if ent.name ~= "ore-siphon" then return end -- just so we end the function as soon as possible if the call is irrelevant
	local surface = ent.surface
	-- it turns out, using the ent itself as a key is a rather bad idea. It can be a value just fine, but not a key.
	global.siphons[ent.unit_number] = {siphonent = ent, recipe = nil, ore = nil, ingredient = nil, table_res = {}, search = true}
	table.insert(global.siphons_index, ent.unit_number)
	log("added: " .. ent.unit_number)
	if global.siphon_range[ent.force.index] == nil then
		global.siphon_range[ent.force.index] = 15
	end
end

function siphonremoved(event)
	local ent = event.entity
	if not ent then return end
	if ent.name ~= "ore-siphon" then return end
	global.siphons[ent.unit_number] = nil
	for _, unitnum in ipairs(global.siphons_index) do
		if unitnum == ent.unit_number then
			table.remove(global.siphons_index, _)
			break
		end
	end
	-- ent.get_inventory(defines.inventory.assembling_machine_input).clear() -- This doesn't work because it fires too late.
end

function siphontick()
	local rnd = math.random()
	-- for _, siphon_data in pairs(global.siphons) do
	-- the +1 is because lua arrays start from 1 instead of 0
	for index = math.floor(game.tick/interval) % batches+1, #global.siphons_index, batches do
		local siphon_unitnumber = global.siphons_index[index]
		local siphon_data
		-- if global.siphons[siphon_unitnumber] == nil then
			-- table.remove(global.siphons_index, index)
			-- goto continue
		-- else
			siphon_data = global.siphons[siphon_unitnumber]
		-- end
		local siphon = siphon_data.siphonent
		local input_inv = siphon.get_inventory(defines.inventory.assembling_machine_input)
		local recipe = siphon.get_recipe()
		if not input_inv.is_empty() or siphon.to_be_deconstructed(siphon.force) then goto continue end
		--if input_inv.get_item_count() >= 50 then goto continue end
		if siphon_data.recipe ~= recipe then
			siphon_data.search = true
		end
		if not recipe then
			siphon_data.recipe = nil
			goto continue
		end

		if siphon_data.search then
			siphon_data.recipe = recipe
			local size = global.siphon_range[siphon.force.index] or 15
			local area = {left_top={x=siphon.position.x-size, y=siphon.position.y-size}, right_bottom={x=siphon.position.x+size, y=siphon.position.y+size}}
			siphon_data.ingredient = siphon_data.recipe.ingredients[1].name
			siphon_data.ore = string.sub(recipe.name, 15)
			siphon_data.table_res = siphon.surface.find_entities_filtered{area = area, name = siphon_data.ore, limit = 400}
			siphon_data.count = #siphon_data.table_res
			siphon_data.search = false
		end
		
		if siphon_data.count > 0 then
			local res, index, valid
			repeat
				index = math.random(siphon_data.count)
				--index = math.floor(rnd * (siphon_data.count + 1))
				res = siphon_data.table_res[index]
				valid = res.valid
				if not valid then
					table.remove(siphon_data.table_res, index)
					siphon_data.count = siphon_data.count - 1
					if siphon_data.count == 0 then
						siphon_data.search = true
						goto continue
					end
				end
			until valid
			
			-- revisit particles in 0.18 maybe.
			-- siphon.surface.create_entity{name=siphon_data.ore .. "-particle", position=res.position, force=siphon.force, raise_built = false, frame_speed = 1, vertical_speed = 0.05, height = 0.05, movement = {0.0,0}}
			
			if not res.initial_amount then -- check if it's infinite
				if res.amount <= insertcount then
					script.raise_event(defines.events.on_resource_depleted, {entity=res})
					res.destroy()
				else
					res.amount = res.amount - insertcount
				end
			end
			local count = insertcount + insertcount * math.floor(siphon.force.mining_drill_productivity_bonus + rnd)
			siphon.insert({name = siphon_data.ingredient, count = count})
		end
		::continue::
	end
end

script.on_event(defines.events.on_research_finished, function(event)
	local research = event.research
	local tech, level = string.match(research.name, "(.+)%-(%d)$")
	if tech == "RS-siphon-range" then
		global.siphon_range[research.force.index] = 15 + 12 * level
		for _, siphon_data in pairs(global.siphons) do
			if #siphon_data.table_res == 0 then
				siphon_data.search = true
			end
		end
	end
end)

script.on_event({defines.events.on_selected_entity_changed, defines.events.on_player_cursor_stack_changed}, function(event)
	local ent = game.players[event.player_index].selected
	--local cursor = game.players[event.player_index].cursor_stack or game.players[event.player_index].cursor_ghost
	if global.highlight then
		rendering.destroy(global.highlight)
		global.hightlight = nil
	end
	local area, size
	-- if cursor and cursor.name == "ore-siphon" then
		-- area = {left_top={x=ent.position.x-size, y=ent.position.y-size}, right_bottom={x=ent.position.x+size, y=ent.position.y+size}}
	if ent and ent.name == "ore-siphon" then
		size = global.siphon_range[ent.force.index] or 15
		area = {left_top={x=ent.position.x-size, y=ent.position.y-size}, right_bottom={x=ent.position.x+size, y=ent.position.y+size}}
	else
		return
	end
	-- vanilla mining highlight RBG: 78, 173, 159
	global.highlight = rendering.draw_rectangle{color={r=78*.25, g=173*.25, b=159*.25, a=65}, filled = true, draw_on_ground = true, surface = ent.surface, left_top = area.left_top, right_bottom = area.right_bottom, players = event.player}
end)

script.on_event({defines.events.on_marked_for_deconstruction, defines.events.on_pre_player_mined_item}, function(event)
	local ent = event.entity
	if not ent or ent.name ~= "ore-siphon" then return end -- just to protect us against wayward scripts, like the build function...
	ent.get_inventory(defines.inventory.assembling_machine_input).clear()
end)

script.on_nth_tick(interval, siphontick)
script.on_event({defines.events.on_built_entity, defines.events.on_robot_built_entity, defines.events.script_raised_built}, siphonbuilt)
script.on_event({defines.events.on_entity_died, defines.events.on_player_mined_entity, defines.events.on_robot_mined_entity, defines.events.script_raised_destroy}, siphonremoved)

-- for debugging
script.on_event(defines.events.on_gui_opened, function(event)
	local ent = event.entity
	if ent and ent.name == "ore-siphon" then
		local found = false
		for _, unitnum in ipairs(global.siphons_index) do
			if unitnum == ent.unit_number then
				found = true
				break
			end
		end
		log(tostring(ent.unit_number) .. ": " .. (tostring(global.siphons[ent.unit_number]) or "nil") .. "; found: " .. tostring(found))
		-- log(serpent.block(global.siphons_index))
	end
end)

