
for _, resource in pairs(data.raw.resource) do
	-- log(serpent.block(resource))
	local reject = false
	-- because angel ores uses 'results' even for single result ores... but we probably should have done this from the start anyways
	if resource.minable.results then
		for _, result in pairs(resource.minable.results) do
			if result.type == "fluid" then
				reject = true
				break
			end
		end
	end
	
	-- VTK deep core mining compatibility block
	if resource.category and string.find(resource.category, "vtk%-deepcore%-mining") then
		reject = true
	end

	-- general mod compatibility block
	local name
	if resource.minable.result then
		name = resource.minable.result
	elseif resource.minable.results and #resource.minable.results == 1 then
		name = resource.minable.results[1].name
	end
	
	if resource.minable.required_fluid
		or not resource.map_color
		or not name
		or not data.raw.item[name]
	then
		reject = true
	end

	--log(resource.name .. ": " .. (name or "nil"))
	if reject then
		log(resource.name .. ": skipping.")
	else
		log(resource.name .. ": adding siphoning recipe.")
		--log(serpent.block(resource))
		--log(resource.name)
		--log(serpent.block(resource.map_color))
		local dustname = name .. "-dust"
		local resourcedust
		if not data.raw.item[dustname] then -- check for mods that have more than one ore with the same result. (We only need 1 dust per result)
			resourcedust = table.deepcopy(data.raw.item[name])
			resourcedust.name = dustname
			resourcedust.localised_name = {"item-name.ore-dust", {"item-name." .. name}}
			resourcedust.localised_description = {"item-description.ore-dust", {"item-name." .. name}}
			resourcedust.order = resourcedust.order .. "-dust"
			if resourcedust.icons then
				for _, icon in pairs(resourcedust.icons) do
					icon.tint = tint_res
				end
			elseif resourcedust.icon then
				resourcedust.icons =  {{icon = resourcedust.icon, tint = tint_res}}
				resourcedust.icon = nil
			end
			resourcedust.stack_size = 10000
			data.raw.item[resourcedust.name] = resourcedust
		end
		--log(serpent.block(data.raw["item"][name]))
		--log(serpent.block(data.raw["item"][resourcedust.name]))
		
		-- angel's compatibility block
		if #resource.map_color == 0 then
			local map_c = {}
			map_c[1] = resource.map_color["r"]
			map_c[2] = resource.map_color["g"]
			map_c[3] = resource.map_color["b"]
			resource.map_color = map_c
		end
		
		local recipe = {
			type = "recipe",
			name = "ore-siphoning-" .. resource.name,
			localised_name = {"recipe-name.ore-siphoning", {"entity-name." .. resource.name}},
			category = "cat-ore-siphon",
			--enabled = false,
			hidden = false,
			ingredients = {{dustname, 1}},
			results = { {type = "item", name = name, amount = 1, probability = 1} },
			energy_required = 1/7.5,
			subgroup = "raw-resource",
			order = "a-".. name,
			hidden_from_player_crafting = true,
			allow_as_intermediate = false,
			crafting_machine_tint =
			{
			  primary = {r = math.min(resource.map_color[1]*1.4, 1.0), g = math.min(resource.map_color[2]*1.4, 1.0), b = math.min(resource.map_color[3]*1.4, 1.0), a = 1.000},
			  secondary = {r = math.min(resource.map_color[1]*1.4, 1.0), g = math.min(resource.map_color[2]*1.4, 1.0), b = math.min(resource.map_color[3]*1.4, 1.0), a = 1.000},
			  tertiary = {r = 0.7, g = 0.7, b = 0.7, a = 1.0},
			  quaternary = {r = math.min(resource.map_color[1]*1.4, 1.0), g = math.min(resource.map_color[2]*1.4, 1.0), b = math.min(resource.map_color[3]*1.4, 1.0), a = 1.000},
			}
		}
		if data.raw.item[name].icons then
			recipe.icons = table.deepcopy(data.raw.item[name].icons)
			-- decided against tinting since the recipe should resemble the final product closely. The ingredients, on the other hand...
			-- for _, icon in pairs(recipe.icons) do
				-- icon.tint = {r=0.8, g=0.8, b=1.0, a=1.0}
			-- end
			recipe.icon_size = data.raw.item[name].icon_size
		elseif data.raw.item[name].icon then
			-- recipe.icons =  {icon = data.raw.item[name].icon, tint = {r=0.8, g=0.8, b=1.0, a=1.0}}
			recipe.icon = data.raw.item[name].icon
			recipe.icon_size = data.raw.item[name].icon_size
		else
			--todo: add a fallback icon
		end
		data:extend({recipe})
	end
end